@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2015 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
@@ -21,16 +21,20 @@
 import threading
 import config
 import cups
-import gobject
-import gtk
-import Queue
+from gi.repository import GObject
+from gi.repository import GLib
+from gi.repository import Gdk
+from gi.repository import Gtk
+import queue
 
 cups.require ("1.9.60")
 
 import authconn
 from debug import *
 import debug
-from gettext import gettext as _
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
+
 
 ######
 ###### An asynchronous libcups API using IPP with a separate worker
@@ -41,13 +45,13 @@ from gettext import gettext as _
 ### This is the worker thread.
 ###
 class _IPPConnectionThread(threading.Thread):
-    def __init__ (self, queue, conn, reply_handler=None, error_handler=None,
+    def __init__ (self, myqueue, conn, reply_handler=None, error_handler=None,
                   auth_handler=None, user=None, host=None, port=None,
                   encryption=None):
                   
         threading.Thread.__init__ (self)
         self.setDaemon (True)
-        self._queue = queue
+        self._queue = myqueue
         self._conn = conn
         self.host = host
         self.port = port
@@ -55,7 +59,7 @@ class _IPPConnectionThread(threading.Thread):
         self._reply_handler = reply_handler
         self._error_handler = error_handler
         self._auth_handler = auth_handler
-        self._auth_queue = Queue.Queue (1)
+        self._auth_queue = queue.Queue(1)
         self.user = user
         self._destroyed = False
         debugprint ("+%s" % self)
@@ -67,11 +71,11 @@ class _IPPConnectionThread(threading.Thread):
         self._auth_queue.put (password)
 
     def run (self):
-        if self.host == None:
+        if self.host is None:
             self.host = cups.getServer ()
-        if self.port == None:
+        if self.port is None:
             self.port = cups.getPort ()
-        if self._encryption == None:
+        if self._encryption is None:
             self._encryption = cups.getEncryption ()
 
         if self.user:
@@ -86,7 +90,7 @@ class _IPPConnectionThread(threading.Thread):
                                     port=self.port,
                                     encryption=self._encryption)
             self._reply (None)
-        except RuntimeError, e:
+        except RuntimeError as e:
             conn = None
             self._error (e)
 
@@ -96,10 +100,14 @@ class _IPPConnectionThread(threading.Thread):
             self.idle = self._queue.empty ()
             item = self._queue.get ()
             debugprint ("Next task: %s" % repr (item))
-            if item == None:
+            if item is None:
                 # Our signal to quit.
                 self._queue.task_done ()
                 break
+            elif self._destroyed:
+                # Just mark all tasks done
+                self._queue.task_done ()
+                continue
 
             self.idle = False
             (fn, args, kwds, rh, eh, ah) = item
@@ -125,7 +133,7 @@ class _IPPConnectionThread(threading.Thread):
 
                     self._queue.task_done ()
                     self._reply (None)
-                except RuntimeError, e:
+                except RuntimeError as e:
                     debugprint ("...failed")
                     self._queue.task_done ()
                     self._error (e)
@@ -144,14 +152,13 @@ class _IPPConnectionThread(threading.Thread):
 
                 debugprint ("...success")
                 self._reply (result)
-            except Exception, e:
-                debugprint ("...failure")
+            except Exception as e:
+                debugprint ("...failure (%s)" % repr (e))
                 self._error (e)
 
             self._queue.task_done ()
 
         debugprint ("Thread exiting")
-        self._destroyed = True
         del self._conn # already destroyed
         del self._reply_handler
         del self._error_handler
@@ -162,46 +169,50 @@ class _IPPConnectionThread(threading.Thread):
 
         cups.setPasswordCB2 (None)
 
+    def stop (self):
+        self._destroyed = True
+        self._queue.put (None)
+
     def _auth (self, prompt, conn=None, method=None, resource=None):
         def prompt_auth (prompt):
-            gtk.gdk.threads_enter ()
-            if conn == None:
+            Gdk.threads_enter ()
+            if conn is None:
                 self._auth_handler (prompt, self._conn)
             else:
                 self._auth_handler (prompt, self._conn, method, resource)
 
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
             return False
 
-        if self._auth_handler == None:
+        if self._auth_handler is None:
             return ""
 
-        gobject.idle_add (prompt_auth, prompt)
+        GLib.idle_add (prompt_auth, prompt)
         password = self._auth_queue.get ()
         return password
 
     def _reply (self, result):
         def send_reply (handler, result):
             if not self._destroyed:
-                gtk.gdk.threads_enter ()
+                Gdk.threads_enter ()
                 handler (self._conn, result)
-                gtk.gdk.threads_leave ()
+                Gdk.threads_leave ()
             return False
 
         if not self._destroyed and self._reply_handler:
-            gobject.idle_add (send_reply, self._reply_handler, result)
+            GLib.idle_add (send_reply, self._reply_handler, result)
 
     def _error (self, exc):
         def send_error (handler, exc):
             if not self._destroyed:
-                gtk.gdk.threads_enter ()
+                Gdk.threads_enter ()
                 handler (self._conn, exc)
-                gtk.gdk.threads_leave ()
+                Gdk.threads_leave ()
             return False
 
         if not self._destroyed and self._error_handler:
             debugprint ("Add %s to idle" % self._error_handler)
-            gobject.idle_add (send_error, self._error_handler, exc)
+            GLib.idle_add (send_error, self._error_handler, exc)
 
 ###
 ### This is the user-visible class.  Although it does not inherit from
@@ -227,7 +238,7 @@ class IPPConnection:
                   encryption=None, parent=None):
         debugprint ("New IPPConnection")
         self._parent = parent
-        self.queue = Queue.Queue ()
+        self.queue = queue.Queue ()
         self.thread = _IPPConnectionThread (self.queue, self,
                                             reply_handler=reply_handler,
                                             error_handler=error_handler,
@@ -259,12 +270,12 @@ class IPPConnection:
             delattr (self, binding)
 
         if self.thread.isAlive ():
-            gobject.timeout_add_seconds (1, self._reap_thread)
+            debugprint ("Stopping worker thread")
+            self.thread.stop ()
+            GLib.timeout_add_seconds (1, self._reap_thread)
 
     def _reap_thread (self):
         if self.thread.idle:
-            debugprint ("Putting None on the task queue")
-            self.queue.put (None)
             self.queue.join ()
             return False
 
@@ -285,13 +296,13 @@ class IPPConnection:
 
     def _call_function (self, fn, *args, **kwds):
         reply_handler = error_handler = auth_handler = False
-        if kwds.has_key ("reply_handler"):
+        if "reply_handler" in kwds:
             reply_handler = kwds["reply_handler"]
             del kwds["reply_handler"]
-        if kwds.has_key ("error_handler"):
+        if "error_handler" in kwds:
             error_handler = kwds["error_handler"]
             del kwds["error_handler"]
-        if kwds.has_key ("auth_handler"):
+        if "auth_handler" in kwds:
             auth_handler = kwds["auth_handler"]
             del kwds["auth_handler"]
 
@@ -338,18 +349,21 @@ class _IPPAuthOperation:
         del self._client_error_handler
 
     def error_handler (self, conn, exc):
-        if self._client_fn == None:
+        if self._client_fn is None:
             # This is the initial "connection" operation, or a
             # subsequent reconnection attempt.
             debugprint ("Connection/reconnection failed")
             return self._reconnect_error (conn, exc)
 
         if self._cancel:
+            debugprint ("%s (_error_handler): canceled so chaining up" % self)
             return self._error (exc)
 
         if self._reconnect:
             self._reconnect = False
             self._reconnected = True
+            debugprint ("%s (_error_handler): reconnecting (as %s)..." %
+                        (self, self._user))
             conn.reconnect (self._user,
                             reply_handler=self._reconnect_reply,
                             error_handler=self._reconnect_error)
@@ -377,6 +391,10 @@ class _IPPAuthOperation:
             return self._error (exc)
 
         # Not authorized.
+        if forbidden:
+            debugprint ("%s (_error_handler): forbidden" % self)
+        else:
+            debugprint ("%s (_error_handler): not authorized" % self)
 
         if (self._try_as_root and
             self._user != 'root' and
@@ -407,13 +425,15 @@ class _IPPAuthOperation:
         authconn.global_authinfocache.remove_auth_info (host=host,
                                                         port=port)
         self._use_password = ''
+        debugprint ("%s (_error_handler): reconnecting (as %s)..." %
+                    (self, self._user))
         conn.reconnect (self._user,
                         reply_handler=self._reconnect_reply,
                         error_handler=self._reconnect_error)
 
     def auth_handler (self, prompt, conn, method=None, resource=None):
         if self._auth_called == False:
-            if self._user == None:
+            if self._user is None:
                 self._user = cups.getUser()
             if self._user:
                 host = conn.thread.host
@@ -446,12 +466,11 @@ class _IPPAuthOperation:
 
         # If we've previously prompted, explain why we're prompting again.
         if self._dialog_shown:
-            d = gtk.MessageDialog (self._conn.parent,
-                                   gtk.DIALOG_MODAL |
-                                   gtk.DIALOG_DESTROY_WITH_PARENT,
-                                   gtk.MESSAGE_ERROR,
-                                   gtk.BUTTONS_CLOSE,
-                                   _("Not authorized"))
+            d = Gtk.MessageDialog (parent=self._conn.parent,
+                                   modal=True, destroy_with_parent=True,
+                                   message_type=Gtk.MessageType.ERROR,
+                                   buttons=Gtk.ButtonsType.CLOSE,
+                                   text=_("Not authorized"))
             d.format_secondary_text (_("The password may be incorrect."))
             d.run ()
             d.destroy ()
@@ -460,18 +479,18 @@ class _IPPAuthOperation:
         if conn.semantic:
             op = conn.semantic.current_operation ()
 
-        if op == None:
+        if op is None:
             d = authconn.AuthDialog (parent=conn.parent)
         else:
             title = _("Authentication (%s)") % op
             d = authconn.AuthDialog (title=title,
                                      parent=conn.parent)
 
-        d.set_prompt (prompt)
-        if self._user == None:
+        d.set_prompt ('')
+        if self._user is None:
             self._user = cups.getUser()
-        d.set_auth_info ([self._user, ''])
-        d.field_grab_focus ('password')
+        d.set_auth_info (['', ''])
+        d.field_grab_focus ('username')
         d.set_keep_above (True)
         d.show_all ()
         d.connect ("response", self._on_auth_dialog_response)
@@ -498,8 +517,8 @@ class _IPPAuthOperation:
         self._dialog = dialog
         dialog.hide ()
 
-        if (response == gtk.RESPONSE_CANCEL or
-            response == gtk.RESPONSE_DELETE_EVENT):
+        if (response == Gtk.ResponseType.CANCEL or
+            response == Gtk.ResponseType.DELETE_EVENT):
             self._cancel = True
             self._conn.set_auth_info ('')
             authconn.global_authinfocache.remove_auth_info (host=self._conn.thread.host,
@@ -524,7 +543,7 @@ class _IPPAuthOperation:
         # so we've reconnected as that user.  Alternatively, the
         # connection has failed and we're retrying.
         debugprint ("Connected as %s" % self._user)
-        if self._client_fn != None:
+        if self._client_fn is not None:
             self.submit_task ()
 
     def _reconnect_error (self, conn, exc):
@@ -537,19 +556,18 @@ class _IPPAuthOperation:
         if conn.semantic:
             op = conn.semantic.current_operation ()
 
-        if op == None:
+        if op is None:
             msg = _("CUPS server error")
         else:
             msg = _("CUPS server error (%s)") % op
 
-        d = gtk.MessageDialog (self._conn.parent,
-                               gtk.DIALOG_MODAL |
-                               gtk.DIALOG_DESTROY_WITH_PARENT,
-                               gtk.MESSAGE_ERROR,
-                               gtk.BUTTONS_NONE,
-                               msg)
+        d = Gtk.MessageDialog (parent=self._conn.parent,
+                               modal=True, destroy_with_parent=True,
+                               message_type=Gtk.MessageType.ERROR,
+                               buttons=Gtk.ButtonsType.NONE,
+                               text=msg)
 
-        if self._client_fn == None and type (exc) == RuntimeError:
+        if self._client_fn is None and type (exc) == RuntimeError:
             # This was a connection failure.
             message = 'service-error-service-unavailable'
         elif type (exc) == cups.IPPError:
@@ -559,25 +577,35 @@ class _IPPAuthOperation:
 
         d.format_secondary_text (_("There was an error during the "
                                    "CUPS operation: '%s'." % message))
-        d.add_buttons (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                       _("Retry"), gtk.RESPONSE_OK)
-        d.set_default_response (gtk.RESPONSE_OK)
+        d.add_buttons (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                       _("Retry"), Gtk.ResponseType.OK)
+        d.set_default_response (Gtk.ResponseType.OK)
         d.connect ("response", self._on_retry_server_error_response)
+        debugprint ("%s (_reconnect_error): presenting error dialog (%s; %s)" %
+                    (self, msg, message))
         d.show ()
 
     def _on_retry_server_error_response (self, dialog, response):
         dialog.destroy ()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
+            debugprint ("%s: got retry response, reconnecting (as %s)..." %
+                        (self, self._conn.thread.user))
             self._conn.reconnect (self._conn.thread.user,
                                   reply_handler=self._reconnect_reply,
                                   error_handler=self._reconnect_error)
         else:
+            debugprint ("%s: got cancel response" % self)
             self._error (cups.IPPError (0, _("Operation canceled")))
 
     def _error (self, exc):
+        debugprint ("%s (_error): handling %s" % (self, repr (exc)))
         if self._client_error_handler:
+            debugprint ("%s (_error): calling %s" %
+                        (self, self._client_error_handler))
             self._client_error_handler (self._conn, exc)
             self._destroy ()
+        else:
+            debugprint ("%s (_error): no client error handler set" % self)
 
 ###
 ### The user-visible class.
@@ -613,13 +641,13 @@ class IPPAuthConnection(IPPConnection):
 
     def _call_function (self, fn, *args, **kwds):
         reply_handler = error_handler = auth_handler = False
-        if kwds.has_key ("reply_handler"):
+        if "reply_handler" in kwds:
             reply_handler = kwds["reply_handler"]
             del kwds["reply_handler"]
-        if kwds.has_key ("error_handler"):
+        if "error_handler" in kwds:
             error_handler = kwds["error_handler"]
             del kwds["error_handler"]
-        if kwds.has_key ("auth_handler"):
+        if "auth_handler" in kwds:
             auth_handler = kwds["auth_handler"]
             del kwds["auth_handler"]
 
@@ -633,22 +661,20 @@ class IPPAuthConnection(IPPConnection):
 
 if __name__ == "__main__":
     # Demo
-    import gtk
     set_debugging (True)
-    gobject.threads_init ()
     class UI:
         def __init__ (self):
-            w = gtk.Window ()
+            w = Gtk.Window ()
             w.connect ("destroy", self.destroy)
-            b = gtk.Button ("Connect")
+            b = Gtk.Button.new_with_label ("Connect")
             b.connect ("clicked", self.connect_clicked)
-            vbox = gtk.VBox ()
-            vbox.pack_start (b)
+            vbox = Gtk.VBox ()
+            vbox.pack_start (b, False, False, 0)
             w.add (vbox)
-            self.get_devices_button = gtk.Button ("Get Devices")
+            self.get_devices_button = Gtk.Button.new_with_label ("Get Devices")
             self.get_devices_button.connect ("clicked", self.get_devices)
             self.get_devices_button.set_sensitive (False)
-            vbox.pack_start (self.get_devices_button)
+            vbox.pack_start (self.get_devices_button, False, False, 0)
             self.conn = None
             w.show_all ()
 
@@ -658,7 +684,7 @@ if __name__ == "__main__":
             except AttributeError:
                 pass
 
-            gtk.main_quit ()
+            Gtk.main_quit ()
 
         def connect_clicked (self, button):
             if self.conn:
@@ -668,11 +694,11 @@ if __name__ == "__main__":
                                            error_handler=self.connect_failed)
 
         def connected (self, conn, result):
-            debugprint ("Success: %s" % result)
+            debugprint ("Success: %s" % repr (result))
             self.get_devices_button.set_sensitive (True)
 
         def connect_failed (self, conn, exc):
-            debugprint ("Exc %s" % exc)
+            debugprint ("Exc %s" % repr (exc))
             self.get_devices_button.set_sensitive (False)
             self.conn.destroy ()
 
@@ -687,7 +713,7 @@ if __name__ == "__main__":
                 debugprint ("Ignoring stale reply")
                 return
 
-            debugprint ("Got devices: %s" % result)
+            debugprint ("Got devices: %s" % repr (result))
             self.get_devices_button.set_sensitive (True)
 
         def get_devices_error (self, conn, exc):
@@ -695,8 +721,8 @@ if __name__ == "__main__":
                 debugprint ("Ignoring stale error")
                 return
 
-            debugprint ("Error getting devices: %s" % exc)
+            debugprint ("Error getting devices: %s" % repr (exc))
             self.get_devices_button.set_sensitive (True)
 
     UI ()
-    gtk.main ()
+    Gtk.main ()
