@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2013, 2014 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -18,58 +18,63 @@
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import threading
+import config
 import cups
 import cupspk
-import gobject
-import gtk
+from gi.repository import GLib
+from gi.repository import Gdk
+from gi.repository import Gtk
 import os
 from errordialogs import *
 from debug import *
-from gettext import gettext as _
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 N_ = lambda x: x
 
 cups.require("1.9.60")
-class AuthDialog(gtk.Dialog):
+class AuthDialog(Gtk.Dialog):
     AUTH_FIELD={'username': N_("Username:"),
                 'password': N_("Password:"),
                 'domain': N_("Domain:")}
 
     def __init__ (self, title=None, parent=None,
-                  flags=gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR,
-                  buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                           gtk.STOCK_OK, gtk.RESPONSE_OK),
-                  auth_info_required=['username', 'password'],
+                  flags=Gtk.DialogFlags.MODAL,
+                  buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                           Gtk.STOCK_OK, Gtk.ResponseType.OK),
+                  auth_info_required=None,
                   allow_remember=False):
-        if title == None:
+        if title is None:
             title = _("Authentication")
-        gtk.Dialog.__init__ (self, title, parent, flags, buttons)
+        if auth_info_required is None:
+            auth_info_required = ['username', 'password']
+        Gtk.Dialog.__init__ (self, title, parent, flags, buttons)
         self.auth_info_required = auth_info_required
-        self.set_default_response (gtk.RESPONSE_OK)
+        self.set_default_response (Gtk.ResponseType.OK)
         self.set_border_width (6)
         self.set_resizable (False)
-        hbox = gtk.HBox (False, 12)
+        hbox = Gtk.HBox.new (False, 12)
         hbox.set_border_width (6)
-        image = gtk.Image ()
-        image.set_from_stock (gtk.STOCK_DIALOG_AUTHENTICATION,
-                              gtk.ICON_SIZE_DIALOG)
+        image = Gtk.Image ()
+        image.set_from_stock (Gtk.STOCK_DIALOG_AUTHENTICATION,
+                              Gtk.IconSize.DIALOG)
         image.set_alignment (0.0, 0.0)
         hbox.pack_start (image, False, False, 0)
-        vbox = gtk.VBox (False, 12)
-        self.prompt_label = gtk.Label ()
+        vbox = Gtk.VBox.new (False, 12)
+        self.prompt_label = Gtk.Label ()
         vbox.pack_start (self.prompt_label, False, False, 0)
 
         num_fields = len (auth_info_required)
-        table = gtk.Table (num_fields, 2)
+        table = Gtk.Table (n_rows=num_fields, n_columns=2)
         table.set_row_spacings (6)
         table.set_col_spacings (6)
 
         self.field_entry = []
         for i in range (num_fields):
             field = auth_info_required[i]
-            label = gtk.Label (_(self.AUTH_FIELD.get (field, field)))
+            label = Gtk.Label (label=_(self.AUTH_FIELD.get (field, field)))
             label.set_alignment (0, 0.5)
             table.attach (label, 0, 1, i, i + 1)
-            entry = gtk.Entry ()
+            entry = Gtk.Entry ()
             entry.set_visibility (field != 'password')
             table.attach (entry, 1, 2, i, i + 1, 0, 0)
             self.field_entry.append (entry)
@@ -77,12 +82,12 @@ class AuthDialog(gtk.Dialog):
         self.field_entry[num_fields - 1].set_activates_default (True)
         vbox.pack_start (table, False, False, 0)
         hbox.pack_start (vbox, False, False, 0)
-        self.vbox.pack_start (hbox)
+        self.vbox.pack_start (hbox, False, False, 0)
 
         if allow_remember:
-            cb = gtk.CheckButton (_("Remember password"))
+            cb = Gtk.CheckButton.new_with_label (_("Remember password"))
             cb.set_active (False)
-            vbox.pack_start (cb)
+            vbox.pack_start (cb, False, False, 0)
             self.remember_checkbox = cb
 
         self.vbox.show_all ()
@@ -99,7 +104,7 @@ class AuthDialog(gtk.Dialog):
             self.field_entry[i].set_text (auth_info[i])
 
     def get_auth_info (self):
-        return map (lambda x: x.get_text (), self.field_entry)
+        return [x.get_text () for x in self.field_entry]
 
     def get_remember_password (self):
         try:
@@ -119,13 +124,13 @@ class _AuthInfoCache:
         self.creds = dict() # by (host,port)
 
     def cache_auth_info (self, data, host=None, port=None):
-        if port == None:
+        if port is None:
             port = 631
 
         self.creds[(host,port)] = data
 
     def lookup_auth_info (self, host=None, port=None):
-        if port == None:
+        if port is None:
             port = 631
 
         try:
@@ -134,7 +139,7 @@ class _AuthInfoCache:
             return None
 
     def remove_auth_info (self, host=None, port=None):
-        if port == None:
+        if port is None:
             port = 631
 
         try:
@@ -147,11 +152,11 @@ global_authinfocache = _AuthInfoCache ()
 class Connection:
     def __init__ (self, parent=None, try_as_root=True, lock=False,
                   host=None, port=None, encryption=None):
-        if host != None:
+        if host is not None:
             cups.setServer (host)
-        if port != None:
+        if port is not None:
             cups.setPort (port)
-        if encryption != None:
+        if encryption is not None:
             cups.setEncryption (encryption)
 
         self._use_password = ''
@@ -169,7 +174,7 @@ class Connection:
         self._connect ()
 
     def _begin_operation (self, operation):
-        debugprint ("%s: Operation += %s" % (self, repr (operation)))
+        debugprint ("%s: Operation += %s" % (self, operation))
         self._operation_stack.append (operation)
 
     def _end_operation (self):
@@ -215,6 +220,9 @@ class Connection:
                 continue
             setattr (self, fname, self._make_binding (fname, fn))
 
+    def _using_polkit (self):
+        return isinstance (self._connection, cupspk.Connection)
+
     def _make_binding (self, fname, fn):
         return lambda *args, **kwds: self._authloop (fname, fn, *args, **kwds)
 
@@ -242,7 +250,8 @@ class Connection:
                         # Authentication failed, but we aren't told that.
                         raise cups.IPPError (cups.IPP_NOT_AUTHORIZED, '')
                 break
-            except cups.IPPError, (e, m):
+            except cups.IPPError as e:
+                (e, m) = e.args
                 if self._use_pk and m == 'pkcancel':
                     raise cups.IPPError (0, _("Operation canceled"))
 
@@ -251,14 +260,16 @@ class Connection:
                                          e == cups.IPP_AUTHENTICATION_CANCELED):
                     self._failed (e == cups.IPP_FORBIDDEN)
                 elif not self._cancel and e == cups.IPP_SERVICE_UNAVAILABLE:
+                    debugprint ("Got IPP_SERVICE_UNAVAILABLE")
+                    debugprint (m)
                     if self._lock:
                         self._gui_event.clear ()
-                        gobject.timeout_add (1, self._ask_retry_server_error, m)
+                        GLib.timeout_add (1, self._ask_retry_server_error, m)
                         self._gui_event.wait ()
                     else:
                         self._ask_retry_server_error (m)
 
-                    if self._retry_response == gtk.RESPONSE_OK:
+                    if self._retry_response == Gtk.ResponseType.OK:
                         debugprint ("retrying operation...")
                         retry = True
                         self._passes -= 1
@@ -270,9 +281,10 @@ class Connection:
                     if self._cancel and not self._cannot_auth:
                         raise cups.IPPError (0, _("Operation canceled"))
 
-                    debugprint ("%s: %s" % (e, m))
+                    debugprint ("%s: %s" % (e, repr (m)))
                     raise
-            except cups.HTTPError, (s,):
+            except cups.HTTPError as e:
+                (s,) = e.args
                 if not self._cancel:
                     self._failed (s == cups.HTTP_FORBIDDEN)
                 else:
@@ -282,28 +294,27 @@ class Connection:
 
     def _ask_retry_server_error (self, message):
         if self._lock:
-            gtk.gdk.threads_enter ()
+            Gdk.threads_enter ()
 
         try:
-            msg = _("CUPS server error (%s)") % self._operation_stack[0]
+            msg = (_("CUPS server error (%s)") % self._operation_stack[0])
         except IndexError:
             msg = _("CUPS server error")
 
-        d = gtk.MessageDialog (self._parent,
-                               gtk.DIALOG_MODAL |
-                               gtk.DIALOG_DESTROY_WITH_PARENT,
-                               gtk.MESSAGE_ERROR,
-                               gtk.BUTTONS_NONE,
-                               msg)
-                               
+        d = Gtk.MessageDialog (parent=self._parent,
+                               modal=True, destroy_with_parent=True,
+                               message_type=Gtk.MessageType.ERROR,
+                               buttons=Gtk.ButtonsType.NONE,
+                               text=msg)
+
         d.format_secondary_text (_("There was an error during the "
-                                   "CUPS operation: '%s'." % message))
-        d.add_buttons (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                       _("Retry"), gtk.RESPONSE_OK)
-        d.set_default_response (gtk.RESPONSE_OK)
+                                   "CUPS operation: '%s'.") % message)
+        d.add_buttons (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                       _("Retry"), Gtk.ResponseType.OK)
+        d.set_default_response (Gtk.ResponseType.OK)
         if self._lock:
             d.connect ("response", self._on_retry_server_error_response)
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
         else:
             self._retry_response = d.run ()
             d.destroy ()
@@ -330,7 +341,7 @@ class Connection:
         self._passes += 1
 
         creds = global_authinfocache.lookup_auth_info (host=self._server, port=self._port)
-        if creds != None:
+        if creds is not None:
             if (creds[0] != 'root' or self._try_as_root):
                 (self._use_user, self._use_password) = creds
             del creds
@@ -402,14 +413,14 @@ class Connection:
         if self._dialog_shown:
             if self._lock:
                 self._gui_event.clear ()
-                gobject.timeout_add (1, self._show_not_authorized_dialog)
+                GLib.timeout_add (1, self._show_not_authorized_dialog)
                 self._gui_event.wait ()
             else:
                 self._show_not_authorized_dialog ()
 
         if self._lock:
             self._gui_event.clear ()
-            gobject.timeout_add (1, self._perform_authentication_with_dialog)
+            GLib.timeout_add (1, self._perform_authentication_with_dialog)
             self._gui_event.wait ()
         else:
             self._perform_authentication_with_dialog ()
@@ -430,12 +441,11 @@ class Connection:
 
     def _show_not_authorized_dialog (self):
         if self._lock:
-            gtk.gdk.threads_enter ()
-        d = gtk.MessageDialog (self._parent,
-                               gtk.DIALOG_MODAL |
-                               gtk.DIALOG_DESTROY_WITH_PARENT,
-                               gtk.MESSAGE_ERROR,
-                               gtk.BUTTONS_CLOSE)
+            Gdk.threads_enter ()
+        d = Gtk.MessageDialog (parent=self._parent,
+                               modal=True, destroy_with_parent=True,
+                               message_type=Gtk.MessageType.ERROR,
+                               buttons=Gtk.ButtonsType.CLOSE)
         d.set_title (_("Not authorized"))
         d.set_markup ('<span weight="bold" size="larger">' +
                       _("Not authorized") + '</span>\n\n' +
@@ -444,7 +454,7 @@ class Connection:
             d.connect ("response", self._on_not_authorized_dialog_response)
             d.show_all ()
             d.show_now ()
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
         else:
             d.run ()
             d.destroy ()
@@ -455,12 +465,12 @@ class Connection:
 
     def _perform_authentication_with_dialog (self):
         if self._lock:
-            gtk.gdk.threads_enter ()
+            Gdk.threads_enter ()
 
         # Prompt.
         if len (self._operation_stack) > 0:
             try:
-                title = _("Authentication (%s)") % self._operation_stack[0]
+                title = (_("Authentication (%s)") % self._operation_stack[0])
             except IndexError:
                 title = _("Authentication")
 
@@ -469,16 +479,16 @@ class Connection:
         else:
             d = AuthDialog (parent=self._parent)
 
-        d.set_prompt (self._prompt)
-        d.set_auth_info ([self._use_user, ''])
-        d.field_grab_focus ('password')
+        d.set_prompt ('')
+        d.set_auth_info (['', ''])
+        d.field_grab_focus ('username')
         d.set_keep_above (True)
         d.show_all ()
         d.show_now ()
         self._dialog_shown = True
         if self._lock:
             d.connect ("response", self._on_authentication_response)
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
         else:
             response = d.run ()
             self._on_authentication_response (d, response)
@@ -493,8 +503,8 @@ class Connection:
                                               port=self._port)
         dialog.destroy ()
 
-        if (response == gtk.RESPONSE_CANCEL or
-            response == gtk.RESPONSE_DELETE_EVENT):
+        if (response == Gtk.ResponseType.CANCEL or
+            response == Gtk.ResponseType.DELETE_EVENT):
             self._cancel = True
 
         if self._lock:
@@ -502,12 +512,12 @@ class Connection:
 
 if __name__ == '__main__':
     # Test it out.
-    gtk.gdk.threads_init ()
+    Gdk.threads_init ()
     from timedops import TimedOperation
     set_debugging (True)
     c = TimedOperation (Connection, args=(None,)).run ()
     debugprint ("Connected")
     c._set_lock (True)
-    print TimedOperation (c.getFile,
+    print(TimedOperation (c.getFile,
                           args=('/admin/conf/cupsd.conf',
-                                '/dev/stdout')).run ()
+                                '/dev/stdout')).run ())
