@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2014 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -17,15 +17,18 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+import config
 import cups
 cups.require("1.9.50")
 import dbus
 import dbus.glib
-import gobject
+from gi.repository import GObject
+from gi.repository import GLib
 import time
 from debug import *
 import pprint
-from gettext import gettext as _
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 import ppdcache
 import statereason
 from statereason import StateReason
@@ -49,56 +52,52 @@ def collect_printer_state_reasons (connection, ppdcache):
     except cups.IPPError:
         return result
 
-    for name, printer in printers.iteritems ():
+    for name, printer in printers.items ():
         reasons = printer["printer-state-reasons"]
         for reason in reasons:
             if reason == "none":
                 break
             if state_reason_is_harmless (reason):
                 continue
-            if not result.has_key (name):
+            if name not in result:
                 result[name] = []
             result[name].append (StateReason (name, reason, ppdcache))
     return result
 
-class Monitor(gobject.GObject):
+class Monitor(GObject.GObject):
     __gsignals__ = {
-        'refresh':               (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  []),
-        'monitor-exited' :       (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  []),
-        'state-reason-added':    (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_PYOBJECT]),
-        'state-reason-removed':  (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_PYOBJECT]),
-        'still-connecting':      (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_PYOBJECT]),
-        'now-connected':         (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_STRING]),
-        'job-added':             (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_INT, gobject.TYPE_STRING,
-                                   gobject.TYPE_PYOBJECT,
-                                   gobject.TYPE_PYOBJECT]),
-        'job-event':             (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_INT, gobject.TYPE_STRING,
-                                   gobject.TYPE_PYOBJECT,
-                                   gobject.TYPE_PYOBJECT]),
-        'job-removed':           (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_INT, gobject.TYPE_STRING,
-                                   gobject.TYPE_PYOBJECT]),
-        'printer-added':         (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_STRING]),
-        'printer-event':         (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_STRING, gobject.TYPE_STRING,
-                                   gobject.TYPE_PYOBJECT]),
-        'printer-removed':       (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_STRING]),
-        'cups-connection-error': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  []),
-        'cups-connection-recovered': (gobject.SIGNAL_RUN_LAST,
-                                      gobject.TYPE_NONE, []),
-        'cups-ipp-error':        (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                                  [gobject.TYPE_INT, gobject.TYPE_STRING])
+        'refresh':               (GObject.SignalFlags.RUN_LAST, None, ()),
+        'monitor-exited' :       (GObject.SignalFlags.RUN_LAST, None, ()),
+        'state-reason-added':    (GObject.SignalFlags.RUN_LAST, None,
+                                  (GObject.TYPE_PYOBJECT,)),
+        'state-reason-removed':  (GObject.SignalFlags.RUN_LAST, None,
+                                  (GObject.TYPE_PYOBJECT,)),
+        'still-connecting':      (GObject.SignalFlags.RUN_LAST, None,
+                                  (GObject.TYPE_PYOBJECT,)),
+        'now-connected':         (GObject.SignalFlags.RUN_LAST, None,
+                                  (str,)),
+        'job-added':             (GObject.SignalFlags.RUN_LAST, None,
+                                  (int, str,
+                                   GObject.TYPE_PYOBJECT,
+                                   GObject.TYPE_PYOBJECT,)),
+        'job-event':             (GObject.SignalFlags.RUN_LAST, None,
+                                  (int, str,
+                                   GObject.TYPE_PYOBJECT,
+                                   GObject.TYPE_PYOBJECT,)),
+        'job-removed':           (GObject.SignalFlags.RUN_LAST, None,
+                                  (int, str,
+                                   GObject.TYPE_PYOBJECT,)),
+        'printer-added':         (GObject.SignalFlags.RUN_LAST, None,
+                                  (str,)),
+        'printer-event':         (GObject.SignalFlags.RUN_LAST, None,
+                                  (str, str,
+                                   GObject.TYPE_PYOBJECT,)),
+        'printer-removed':       (GObject.SignalFlags.RUN_LAST, None,
+                                  (str,)),
+        'cups-connection-error': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'cups-connection-recovered': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'cups-ipp-error':        (GObject.SignalFlags.RUN_LAST, None,
+                                  (int, str,))
         }
 
     # Monitor jobs and printers.
@@ -108,7 +107,7 @@ class Monitor(gobject.GObject):
     def __init__(self, bus=None, my_jobs=True,
                  specific_dests=None, monitor_jobs=True, host=None,
                  port=None, encryption=None):
-        gobject.GObject.__init__ (self)
+        GObject.GObject.__init__ (self)
         self.my_jobs = my_jobs
         self.specific_dests = specific_dests
         self.monitor_jobs = monitor_jobs
@@ -141,7 +140,7 @@ class Monitor(gobject.GObject):
         self.received_any_dbus_signals = False
         self.update_timer = None
 
-        if bus == None:
+        if bus is None:
             try:
                 bus = dbus.SystemBus ()
             except dbus.exceptions.DBusException:
@@ -149,7 +148,7 @@ class Monitor(gobject.GObject):
                 pass
 
         self.bus = bus
-        if bus != None:
+        if bus is not None:
             bus.add_signal_receiver (self.handle_dbus_signal,
                                      path=self.DBUS_PATH,
                                      dbus_interface=self.DBUS_IFACE)
@@ -178,17 +177,17 @@ class Monitor(gobject.GObject):
                 pass
             cups.setUser (user)
 
-        if self.bus != None:
+        if self.bus is not None:
             self.bus.remove_signal_receiver (self.handle_dbus_signal,
                                              path=self.DBUS_PATH,
                                              dbus_interface=self.DBUS_IFACE)
 
-        timers = self.connecting_timers.values ()
+        timers = list(self.connecting_timers.values ())
         for timer in [self.update_timer, self.fetch_jobs_timer]:
             if timer:
                 timers.append (timer)
         for timer in timers:
-            gobject.source_remove (timer)
+            GLib.source_remove (timer)
 
         self.emit ('monitor-exited')
 
@@ -199,12 +198,12 @@ class Monitor(gobject.GObject):
         """Timer callback to check on connecting-to-device reasons."""
         if not self.process_pending_events:
             # Defer the timer by setting a new one.
-            timer = gobject.timeout_add (200, self.check_still_connecting,
-                                         printer)
+            timer = GLib.timeout_add (200, self.check_still_connecting,
+                                      printer)
             self.connecting_timers[printer] = timer
             return False
 
-        if self.connecting_timers.has_key (printer):
+        if printer in self.connecting_timers:
             del self.connecting_timers[printer]
 
         debugprint ("Still-connecting timer fired for `%s'" % printer)
@@ -219,13 +218,12 @@ class Monitor(gobject.GObject):
         time_now = time.time ()
         connecting_to_device = {}
         trouble = False
-        for printer, reasons in self.printer_state_reasons.iteritems ():
+        for printer, reasons in self.printer_state_reasons.items ():
             connected = True
             for reason in reasons:
                 if reason.get_reason () == "connecting-to-device":
                     have_processing_job = False
-                    for job, data in \
-                            printer_jobs.get (printer, {}).iteritems ():
+                    for job, data in printer_jobs.get (printer, {}).items ():
                         state = data.get ('job-state',
                                           cups.IPP_JOB_CANCELED)
                         if state == cups.IPP_JOB_PROCESSING:
@@ -247,9 +245,9 @@ class Monitor(gobject.GObject):
                             if printer not in self.still_connecting:
                                 self.still_connecting.add (printer)
                                 self.emit ('still-connecting', reason)
-                            if self.connecting_timers.has_key (printer):
-                                gobject.source_remove (self.connecting_timers
-                                                       [printer])
+                            if printer in self.connecting_timers:
+                                GLib.source_remove (self.connecting_timers
+                                                    [printer])
                                 del self.connecting_timers[printer]
                                 debugprint ("Stopped connecting timer "
                                             "for `%s'" % printer)
@@ -257,19 +255,19 @@ class Monitor(gobject.GObject):
                     connected = False
                     break
 
-            if connected and self.connecting_timers.has_key (printer):
-                gobject.source_remove (self.connecting_timers[printer])
+            if connected and printer in self.connecting_timers:
+                GLib.source_remove (self.connecting_timers[printer])
                 del self.connecting_timers[printer]
                 debugprint ("Stopped connecting timer for `%s'" % printer)
 
         # Clear any previously-notified errors that are now fine.
         remove = set()
         for printer in self.still_connecting:
-            if not connecting_to_device.has_key (printer):
+            if printer not in connecting_to_device:
                 remove.add (printer)
                 self.emit ('now-connected', printer)
-                if self.connecting_timers.has_key (printer):
-                    gobject.source_remove (self.connecting_timers[printer])
+                if printer in self.connecting_timers:
+                    GLib.source_remove (self.connecting_timers[printer])
                     del self.connecting_timers[printer]
                     debugprint ("Stopped connecting timer for `%s'" % printer)
 
@@ -278,25 +276,26 @@ class Monitor(gobject.GObject):
 
     def check_state_reasons(self, my_printers=set(), printer_jobs={}):
         # Look for any new reasons since we last checked.
-        old_reasons_seen_keys = self.reasons_seen.keys ()
+        old_reasons_seen_keys = list(self.reasons_seen.keys ())
         reasons_now = set()
-        for printer, reasons in self.printer_state_reasons.iteritems ():
+        for printer, reasons in self.printer_state_reasons.items ():
             for reason in reasons:
                 tuple = reason.get_tuple ()
                 printer = reason.get_printer ()
                 reasons_now.add (tuple)
-                if not self.reasons_seen.has_key (tuple):
+                if tuple not in self.reasons_seen:
                     # New reason.
-                    self.emit ('state-reason-added', reason)
+                    GLib.idle_add (lambda x:
+                                       self.emit ('state-reason-added', x),
+                                   reason)
                     self.reasons_seen[tuple] = reason
 
                 if (reason.get_reason () == "connecting-to-device" and
-                    not self.connecting_to_device.has_key (printer)):
+                    printer not in self.connecting_to_device):
                     # First time we've seen this.
 
                     have_processing_job = False
-                    for job, data in \
-                            printer_jobs.get (printer, {}).iteritems ():
+                    for job, data in printer_jobs.get (printer, {}).items ():
                         state = data.get ('job-state',
                                           cups.IPP_JOB_CANCELED)
                         if state == cups.IPP_JOB_PROCESSING:
@@ -304,7 +303,7 @@ class Monitor(gobject.GObject):
                             break
 
                     if have_processing_job:
-                        t = gobject.timeout_add_seconds (
+                        t = GLib.timeout_add_seconds (
                             (1 + CONNECTING_TIMEOUT),
                             self.check_still_connecting,
                             printer)
@@ -318,21 +317,23 @@ class Monitor(gobject.GObject):
                             debugprint (pprint.pformat (printer_jobs))
 
         self.update_connecting_devices (printer_jobs)
-        items = self.reasons_seen.keys ()
+        items = list(self.reasons_seen.keys ())
         for tuple in items:
             if not tuple in reasons_now:
                 # Reason no longer present.
                 reason = self.reasons_seen[tuple]
                 del self.reasons_seen[tuple]
-                self.emit ('state-reason-removed', reason)
+                GLib.idle_add (lambda x: self.emit ('state-reason-removed', x),
+                               reason)
 
     def get_notifications(self):
+        if self.update_timer:
+            GLib.source_remove (self.update_timer)
+
+        self.update_timer = None
         if not self.process_pending_events:
             # Defer the timer callback.
-            if self.update_timer:
-                gobject.source_remove (self.update_timer)
-
-            self.update_timer = gobject.timeout_add (200,
+            self.update_timer = GLib.timeout_add (200,
                                                      self.get_notifications)
             debugprint ("Deferred get_notifications by 200ms")
             return False
@@ -351,7 +352,8 @@ class Monitor(gobject.GObject):
                                                         [self.sub_seq + 1])
                 except AttributeError:
                     notifications = c.getNotifications ([self.sub_id])
-            except cups.IPPError, (e, m):
+            except cups.IPPError as e:
+                (e, m) = e.args
                 cups.setUser (user)
                 if e == cups.IPP_NOT_FOUND:
                     # Subscription lease has expired.
@@ -396,14 +398,14 @@ class Monitor(gobject.GObject):
 
                 elif nse == 'printer-deleted' and name in self.printers:
                     self.printers.remove (name)
-                    items = self.reasons_seen.keys ()
+                    items = list(self.reasons_seen.keys ())
                     for tuple in items:
                         if tuple[1] == name:
                             reason = self.reasons_seen[tuple]
                             del self.reasons_seen[tuple]
                             self.emit ('state-reason-removed', reason)
                             
-                    if self.printer_state_reasons.has_key (name):
+                    if name in self.printer_state_reasons:
                         del self.printer_state_reasons[name]
 
                     self.emit ('printer-removed', name)
@@ -432,9 +434,9 @@ class Monitor(gobject.GObject):
             jobid = event['notify-job-id']
             if (nse == 'job-created' or
                 (nse == 'job-state-changed' and
-                 not jobs.has_key (jobid) and
+                 jobid not in jobs and
                  event['job-state'] == cups.IPP_JOB_PROCESSING)):
-                if (self.specific_dests != None and
+                if (self.specific_dests is not None and
                     event['printer-name'] not in self.specific_dests):
                     continue
 
@@ -447,7 +449,8 @@ class Monitor(gobject.GObject):
                     jobs[jobid] = attrs
                 except KeyError:
                     jobs[jobid] = {'job-k-octets': 0}
-                except cups.IPPError, (e, m):
+                except cups.IPPError as e:
+                    (e, m) = e.args
                     self.emit ('cups-ipp-error', e, m)
                     jobs[jobid] = {'job-k-octets': 0}
 
@@ -468,10 +471,16 @@ class Monitor(gobject.GObject):
             except KeyError:
                 continue
 
+            if (self.specific_dests is not None and
+                event['printer-name'] not in self.specific_dests):
+                del jobs[jobid]
+                self.emit ('job-removed', jobid, nse, event)
+                continue
+
             for attribute in ['job-state',
                               'job-name']:
                 job[attribute] = event[attribute]
-            if event.has_key ('notify-printer-uri'):
+            if 'notify-printer-uri' in event:
                 job['job-printer-uri'] = event['notify-printer-uri']
 
             self.emit ('job-event', jobid, nse, event, job.copy ())
@@ -484,12 +493,9 @@ class Monitor(gobject.GObject):
         # Update again when we're told to.  If we're getting CUPS
         # D-Bus signals, however, rely on those instead.
         if not self.received_any_dbus_signals:
-            if self.update_timer:
-                gobject.source_remove (self.update_timer)
-
             interval = notifications['notify-get-interval']
-            t = gobject.timeout_add_seconds (interval,
-                                             self.get_notifications)
+            t = GLib.timeout_add_seconds (interval,
+                                          self.get_notifications)
             debugprint ("Next notifications fetch in %ds" % interval)
             self.update_timer = t
 
@@ -499,7 +505,7 @@ class Monitor(gobject.GObject):
         debugprint ("refresh")
 
         self.emit ('refresh')
-        if which_jobs != None:
+        if which_jobs is not None:
             self.which_jobs = which_jobs
 
         user = cups.getUser ()
@@ -509,19 +515,22 @@ class Monitor(gobject.GObject):
                                  port=self.port,
                                  encryption=self.encryption)
         except RuntimeError:
-            self.emit ('cups-connection-error')
+            GLib.idle_add (self.emit, 'cups-connection-error')
             cups.setUser (user)
             return
 
         if self.sub_id != -1:
             try:
                 c.cancelSubscription (self.sub_id)
-            except cups.IPPError, (e, m):
-                self.emit ('cups-ipp-error', e, m)
+            except cups.IPPError as e:
+                (e, m) = e.args
+                GLib.idle_add (lambda e, m: self.emit ('cups-ipp-error', e, m),
+                               e, m)
 
             if self.update_timer:
-                gobject.source_remove (self.update_timer)
+                GLib.source_remove (self.update_timer)
 
+            self.update_timer = None
             debugprint ("Canceled subscription %d" % self.sub_id)
 
         try:
@@ -530,6 +539,7 @@ class Monitor(gobject.GObject):
             pass
 
         events = ["printer-added",
+                  "printer-modified",
                   "printer-deleted",
                   "printer-state-changed"]
         if self.monitor_jobs:
@@ -543,13 +553,18 @@ class Monitor(gobject.GObject):
             self.sub_id = c.createSubscription ("/", events=events)
             debugprint ("Created subscription %d, events=%s" % (self.sub_id,
                                                                 repr (events)))
-        except cups.IPPError, (e, m):
-            self.emit ('cups-ipp-error', e, m)
+        except cups.IPPError as e:
+            (e, m) = e.args
+            GLib.idle_add (lambda e, m: self.emit ('cups-ipp-error', e, m),
+                           e, m)
 
         cups.setUser (user)
 
         if self.sub_id != -1:
-            self.update_timer = gobject.timeout_add_seconds (
+            if self.update_timer:
+                GLib.source_remove (self.update_timer)
+
+            self.update_timer = GLib.timeout_add_seconds (
                 MIN_REFRESH_INTERVAL,
                 self.get_notifications)
             debugprint ("Next notifications fetch in %ds" %
@@ -560,7 +575,7 @@ class Monitor(gobject.GObject):
             if self.which_jobs not in ['all', 'completed']:
                 # Filter out completed jobs.
                 filtered = {}
-                for jobid, job in jobs.iteritems ():
+                for jobid, job in jobs.items ():
                     if job.get ('job-state',
                                 cups.IPP_JOB_CANCELED) < cups.IPP_JOB_CANCELED:
                         filtered[jobid] = job
@@ -568,9 +583,9 @@ class Monitor(gobject.GObject):
 
             self.fetch_first_job_id = 1
             if self.fetch_jobs_timer:
-                gobject.source_remove (self.fetch_jobs_timer)
-            self.fetch_jobs_timer = gobject.timeout_add (5, self.fetch_jobs,
-                                                         refresh_all)
+                GLib.source_remove (self.fetch_jobs_timer)
+            self.fetch_jobs_timer = GLib.timeout_add (5, self.fetch_jobs,
+                                                      refresh_all)
         else:
             jobs = {}
 
@@ -579,14 +594,16 @@ class Monitor(gobject.GObject):
             self.printer_state_reasons = r
             dests = c.getPrinters ()
             self.printers = set(dests.keys ())
-        except cups.IPPError, (e, m):
-            self.emit ('cups-ipp-error', e, m)
+        except cups.IPPError as e:
+            (e, m) = e.args
+            GLib.idle_add (lambda e, m: self.emit ('cups-ipp-error', e, m),
+                           e, m)
             return
         except RuntimeError:
-            self.emit ('cups-connection-error')
+            GLib.idle_add (self.emit, 'cups-connection-error')
             return
 
-        if self.specific_dests != None:
+        if self.specific_dests is not None:
             for jobid in jobs.keys ():
                 uri = jobs[jobid].get('job-printer-uri', '/')
                 i = uri.rfind ('/')
@@ -596,9 +613,11 @@ class Monitor(gobject.GObject):
 
         self.set_process_pending (False)
         for printer in self.printers:
-            self.emit ('printer-added', printer)
-        for jobid, job in jobs.iteritems ():
-            self.emit ('job-added', jobid, '', {}, job)
+            GLib.idle_add (lambda x: self.emit ('printer-added', x), printer)
+        for jobid, job in jobs.items ():
+            GLib.idle_add (lambda jobid, job:
+                                  self.emit ('job-added', jobid, '', {}, job),
+                           jobid, job)
         self.update_jobs (jobs)
         self.jobs = jobs
         self.set_process_pending (True)
@@ -635,7 +654,8 @@ class Monitor(gobject.GObject):
                                  first_job_id=self.fetch_first_job_id,
                                  limit=limit,
                                  requested_attributes=r)
-        except cups.IPPError, (e, m):
+        except cups.IPPError as e:
+            (e, m) = e.args
             self.emit ('cups-ipp-error', e, m)
             self.fetch_jobs_timer = None
             cups.setUser (user)
@@ -646,7 +666,7 @@ class Monitor(gobject.GObject):
         debugprint ("Got %s jobs, asked for %s" % (got, limit))
 
         jobs = self.jobs.copy ()
-        jobids = fetched.keys ()
+        jobids = list(fetched.keys ())
         jobids.sort ()
         if got > 0:
             last_jobid = jobids[got - 1]
@@ -656,17 +676,17 @@ class Monitor(gobject.GObject):
                 debugprint ("That's not what we asked for!")
         else:
             last_jobid = self.fetch_first_job_id + limit - 1
-        for jobid in xrange (self.fetch_first_job_id, last_jobid + 1):
+        for jobid in range (self.fetch_first_job_id, last_jobid + 1):
             try:
                 job = fetched[jobid]
-                if self.specific_dests != None:
+                if self.specific_dests is not None:
                     uri = job.get('job-printer-uri', '/')
                     i = uri.rfind ('/')
                     printer = uri[i + 1:]
                     if printer not in self.specific_dests:
                         raise KeyError
 
-                if jobs.has_key (jobid):
+                if jobid in jobs:
                     n = 'job-event'
                 else:
                     n = 'job-added'
@@ -675,11 +695,11 @@ class Monitor(gobject.GObject):
                 self.emit (n, jobid, '', {}, job.copy ())
             except KeyError:
                 # No job by that ID.
-                if jobs.has_key (jobid):
+                if jobid in jobs:
                     del jobs[jobid]
                     self.emit ('job-removed', jobid, '', {})
 
-        jobids = jobs.keys ()
+        jobids = list(jobs.keys ())
         jobids.sort ()
         if got < limit:
             trim = False
@@ -703,19 +723,19 @@ class Monitor(gobject.GObject):
         # Remember where we got up to and run this timer again.
         next = jobid + 1
 
-        while not refresh_all and self.jobs.has_key (next):
+        while not refresh_all and next in self.jobs:
             next += 1
 
         self.fetch_first_job_id = next
         return True
 
     def sort_jobs_by_printer (self, jobs=None):
-        if jobs == None:
+        if jobs is None:
             jobs = self.jobs
 
         my_printers = set()
         printer_jobs = {}
-        for job, data in jobs.iteritems ():
+        for job, data in jobs.items ():
             state = data.get ('job-state', cups.IPP_JOB_CANCELED)
             if state >= cups.IPP_JOB_CANCELED:
                 continue
@@ -725,7 +745,7 @@ class Monitor(gobject.GObject):
                 continue
             printer = uri[i + 1:]
             my_printers.add (printer)
-            if not printer_jobs.has_key (printer):
+            if printer not in printer_jobs:
                 printer_jobs[printer] = {}
             printer_jobs[printer][job] = data
 
@@ -738,9 +758,9 @@ class Monitor(gobject.GObject):
 
     def update(self):
         if self.update_timer:
-            gobject.source_remove (self.update_timer)
+            GLib.source_remove (self.update_timer)
 
-        self.update_timer = gobject.timeout_add (200, self.get_notifications)
+        self.update_timer = GLib.timeout_add (200, self.get_notifications)
         debugprint ("Next notifications fetch in 200ms (update called)")
 
     def handle_dbus_signal(self, *args):
@@ -749,8 +769,6 @@ class Monitor(gobject.GObject):
         if not self.received_any_dbus_signals:
             self.received_any_dbus_signals = True
 
-gobject.type_register (Monitor)
-
 if __name__ == '__main__':
     class SignalWatcher:
         def __init__ (self, monitor):
@@ -771,49 +789,49 @@ if __name__ == '__main__':
             monitor.connect ('cups-ipp-error', self.on_cups_ipp_error)
 
         def on_monitor_exited (self, obj):
-            print "*%s: monitor exited" % obj
+            print("*%s: monitor exited" % obj)
 
         def on_state_reason_added (self, obj, reason):
-            print "*%s: +%s" % (obj, reason)
+            print("*%s: +%s" % (obj, reason))
 
         def on_state_reason_removed (self, obj, reason):
-            print "*%s: -%s" % (obj, reason)
+            print("*%s: -%s" % (obj, reason))
 
         def on_still_connecting (self, obj, reason):
-            print "*%s: still connecting: %s" % (obj, reason)
+            print("*%s: still connecting: %s" % (obj, reason))
 
         def on_now_connected (self, obj, printer):
-            print "*%s: now connected: %s" % (obj, printer)
+            print("*%s: now connected: %s" % (obj, printer))
 
         def on_job_added (self, obj, jobid, eventname, event, jobdata):
-            print "*%s: job %d added" % (obj, jobid)
+            print("*%s: job %d added" % (obj, jobid))
 
         def on_job_event (self, obj, jobid, eventname, event, jobdata):
-            print "*%s: job %d event: %s" % (obj, jobid, event)
+            print("*%s: job %d event: %s" % (obj, jobid, event))
 
         def on_job_removed (self, obj, jobid, eventname, event):
-            print "*%s: job %d removed (%s)"% (obj, jobid, eventname)
+            print("*%s: job %d removed (%s)"% (obj, jobid, eventname))
 
         def on_printer_added (self, obj, name):
-            print "*%s: printer added: %s" % (obj, name)
+            print("*%s: printer added: %s" % (obj, name))
 
         def on_printer_event (self, obj, name, eventname, event):
-            print "*%s: printer event: %s: %s" % (obj, name, eventname)
+            print("*%s: printer event: %s: %s" % (obj, name, eventname))
 
         def on_printer_removed (self, obj, name):
-            print "*%s: printer %s removed" % (obj, name)
+            print("*%s: printer %s removed" % (obj, name))
 
         def on_cups_connection_error (self, obj):
-            print "*%s: cups connection error" % obj
+            print("*%s: cups connection error" % obj)
 
         def on_cups_ipp_error (self, obj, err, errstring):
-            print "*%s: IPP error (%d): %s" % (obj, err, errstring)
+            print("*%s: IPP error (%d): %s" % (obj, err, errstring))
 
     set_debugging (True)
     m = Monitor ()
     SignalWatcher (m)
     m.refresh ()
-    loop = gobject.MainLoop ()
+    loop = GObject.MainLoop ()
     try:
         loop.run ()
     finally:
