@@ -1,9 +1,9 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2012 Red Hat, Inc.
+## Copyright (C) 2008, 2012 Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@
 
 import locale
 
-from base import *
+from gi.repository import Gtk
+
+from .base import *
 
 class Locale(Question):
     def __init__ (self, troubleshooter):
@@ -31,25 +33,24 @@ class Locale(Question):
                                     "not the printer's default page size.  "
                                     "If this is not intentional it may cause "
                                     "alignment problems."))
-
-        table = gtk.Table (2, 2)
+        table = Gtk.Table (n_rows=2, n_columns=2)
         table.set_row_spacings (6)
         table.set_col_spacings (6)
-        page.pack_start (table)
-        self.printer_page_size = gtk.Label ()
+        page.pack_start (table, False, False, 0)
+        self.printer_page_size = Gtk.Label ()
         self.printer_page_size.set_alignment (0, 0)
-        self.job_page_size = gtk.Label ()
+        self.job_page_size = Gtk.Label ()
         self.job_page_size.set_alignment (0, 0)
-        label = gtk.Label (_("Print job page size:"))
+        label = Gtk.Label(label=_("Print job page size:"))
         label.set_alignment (0, 0)
-        table.attach (label, 0, 1, 0, 1, xoptions=gtk.FILL, yoptions=0)
+        table.attach (label, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
         table.attach (self.job_page_size, 1, 2, 0, 1,
-                      xoptions=gtk.FILL, yoptions=0)
-        label = gtk.Label (_("Printer page size:"))
+                      xoptions=Gtk.AttachOptions.FILL, yoptions=0)
+        label = Gtk.Label(label=_("Printer page size:"))
         label.set_alignment (0, 0)
-        table.attach (label, 0, 1, 1, 2, xoptions=gtk.FILL, yoptions=0)
+        table.attach (label, 0, 1, 1, 2, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
         table.attach (self.printer_page_size, 1, 2, 1, 2,
-                      xoptions=gtk.FILL, yoptions=0)
+                      xoptions=Gtk.AttachOptions.FILL, yoptions=0)
         troubleshooter.new_page (page, self)
 
     def display (self):
@@ -60,10 +61,22 @@ class Locale(Question):
         self.answers['user_locale_ctype'] = ctype
 
         try:
-            i18n = file ("/etc/sysconfig/i18n").readlines ()
-            for line in i18n:
-                if line.startswith ("LANG="):
-                    system_lang = line[5:].strip ('\n"')
+            system_lang = None
+            conf = None
+            for conffile in ["/etc/locale.conf", "/etc/sysconfig/i18n"]:
+                try:
+                    conf = open (conffile).readlines ()
+                except IOError:
+                    continue
+
+            if conf is not None:
+                for line in conf:
+                    if line.startswith("LC_PAPER="):
+                        system_lang = line[9:].strip ('\n"')
+                    elif system_lang is None and line.startswith ("LANG="):
+                        system_lang = line[5:].strip ('\n"')
+
+                if system_lang is not None:
                     dot = system_lang.find ('.')
                     if dot != -1:
                         system_lang = system_lang[:dot]
@@ -75,8 +88,8 @@ class Locale(Question):
         printer_page_size = None
         try:
             ppd_defs = self.troubleshooter.answers['cups_printer_ppd_defaults']
-            for group, options in ppd_defs.iteritems ():
-                if options.has_key ("PageSize"):
+            for group, options in ppd_defs.items ():
+                if "PageSize" in options:
                     printer_page_size = options["PageSize"]
                     break
 
@@ -93,11 +106,11 @@ class Locale(Question):
             job_status = []
 
         self.answers['printer_page_size'] = printer_page_size
-        if printer_page_size != None:
+        if printer_page_size is not None:
             job_page_size = None
             for (test, jobid, printer, doc, status, attrs) in job_status:
                 if test:
-                    if attrs.has_key ("PageSize"):
+                    if "PageSize" in attrs:
                         job_page_size = attrs["PageSize"]
                         self.answers['job_page_size'] = job_page_size
                         if job_page_size != printer_page_size:
